@@ -13,7 +13,13 @@ from typing import Any, Generator, Optional, Tuple, Union
 import pytest
 
 import codespell_lib as cs_
-from codespell_lib._codespell import EX_DATAERR, EX_OK, EX_USAGE, uri_regex_def
+from codespell_lib._codespell import (
+    EX_CONFIG,
+    EX_DATAERR,
+    EX_OK,
+    EX_USAGE,
+    uri_regex_def,
+)
 
 
 def test_constants() -> None:
@@ -21,6 +27,7 @@ def test_constants() -> None:
     assert EX_OK == 0
     assert EX_USAGE == 64
     assert EX_DATAERR == 65
+    assert EX_CONFIG == 78
 
 
 class MainWrapper:
@@ -34,7 +41,7 @@ class MainWrapper:
     ) -> Union[int, Tuple[int, str, str]]:
         args = tuple(str(arg) for arg in args)
         if count:
-            args = ("--count",) + args
+            args = ("--count", *args)
         code = cs_.main(*args)
         frame = inspect.currentframe()
         assert frame is not None
@@ -42,7 +49,7 @@ class MainWrapper:
         assert frame is not None
         capsys = frame.f_locals["capsys"]
         stdout, stderr = capsys.readouterr()
-        assert code in (EX_OK, EX_USAGE, EX_DATAERR)
+        assert code in (EX_OK, EX_USAGE, EX_DATAERR, EX_CONFIG)
         if code == EX_DATAERR:  # have some misspellings
             code = int(stderr.split("\n")[-2])
         elif code == EX_OK and count:
@@ -69,8 +76,7 @@ def run_codespell(
         encoding="utf-8",
         check=False,
     )
-    count = int(proc.stderr.split("\n")[-2])
-    return count
+    return int(proc.stderr.split("\n")[-2])
 
 
 def test_command(tmp_path: Path) -> None:
@@ -152,7 +158,8 @@ def test_basic(
     assert isinstance(result, tuple)
     code, stdout, stderr = result
     assert code == 0
-    assert not stdout and not stderr
+    assert not stdout
+    assert not stderr
     assert cs.main(tmp_path) == 0
 
     # empty directory
@@ -170,9 +177,9 @@ def test_default_word_parsing(
     assert cs.main(fname) == 1, "bad"
 
     fname = tmp_path / "apostrophe"
-    fname.write_text("woudn't\n", encoding="utf-8")  # U+0027 (')
+    fname.write_text("woudn't\n", encoding="utf-8")  # U+0027
     assert cs.main(fname) == 1, "misspelling containing typewriter apostrophe U+0027"
-    fname.write_text("woudn’t\n", encoding="utf-8")  # U+2019 (’)
+    fname.write_text("woudn’t\n", encoding="utf-8")  # U+2019  # noqa: RUF001
     assert cs.main(fname) == 1, "misspelling containing typographic apostrophe U+2019"
 
 
@@ -293,7 +300,8 @@ def test_summary(
     assert isinstance(result, tuple)
     code, stdout, stderr = result
     assert code == 0
-    assert not stdout and not stderr, "no output"
+    assert not stdout
+    assert not stderr, "no output"
     result = cs.main(fname, "--summary", std=True)
     assert isinstance(result, tuple)
     code, stdout, stderr = result
@@ -318,11 +326,73 @@ def test_ignore_dictionary(
 ) -> None:
     """Test ignore dictionary functionality."""
     bad_name = tmp_path / "bad.txt"
-    bad_name.write_text("1 abandonned 1\n2 abandonned 2\nabondon\n")
-    assert cs.main(bad_name) == 3
+    bad_name.write_text(
+        "1 abandonned 1\n"
+        "2 abandonned 2\n"
+        "3 abandonned 3\r\n"
+        "4 abilty 4\n"
+        "5 abilty 5\n"
+        "6 abilty 6\r\n"
+        "7 ackward 7\n"
+        "8 ackward 8\n"
+        "9 ackward 9\r\n"
+        "abondon\n"
+    )
+    assert cs.main(bad_name) == 10
     fname = tmp_path / "ignore.txt"
-    fname.write_text("abandonned\n")
+    fname.write_text("abandonned\nabilty\r\nackward")
     assert cs.main("-I", fname, bad_name) == 1
+    # missing file in ignore list
+    fname_missing = tmp_path / "missing.txt"
+    result = cs.main("-I", fname_missing, bad_name, std=True)
+    assert isinstance(result, tuple)
+    code, _, stderr = result
+    assert code == EX_USAGE
+    assert "ERROR:" in stderr
+    # comma-separated list of files
+    fname_dummy1 = tmp_path / "dummy1.txt"
+    fname_dummy1.touch()
+    fname_dummy2 = tmp_path / "dummy2.txt"
+    fname_dummy2.touch()
+    assert cs.main("-I", fname_dummy1, "-I", fname, "-I", fname_dummy2, bad_name) == 1
+    assert cs.main("-I", f"{fname_dummy1},{fname},{fname_dummy2}", bad_name) == 1
+
+
+def test_ignore_words_with_cases(
+    tmp_path: Path,
+    capsys: pytest.CaptureFixture[str],
+) -> None:
+    """Test case-sensitivity implemented for -I and -L options in #3272."""
+    bad_name = tmp_path / "MIS.txt"
+    bad_name.write_text(
+        "1 MIS (Management Information System) 1\n"
+        "2 Les Mis (1980 musical) 2\n"
+        "3 mis 3\n"
+    )
+    assert cs.main(bad_name) == 3
+    assert cs.main(bad_name, "-f") == 4
+    fname = tmp_path / "ignore.txt"
+
+    fname.write_text("miS")
+    assert cs.main("-I", fname, bad_name) == 3
+    assert cs.main("-LmiS", bad_name) == 3
+    assert cs.main("-I", fname, "-f", bad_name) == 4
+    assert cs.main("-LmiS", "-f", bad_name) == 4
+    fname.write_text("MIS")
+    assert cs.main("-I", fname, bad_name) == 2
+    assert cs.main("-LMIS", bad_name) == 2
+    assert cs.main("-I", fname, "-f", bad_name) == 2
+    assert cs.main("-LMIS", "-f", bad_name) == 2
+    fname.write_text("MIS\nMis")
+    assert cs.main("-I", fname, bad_name) == 1
+    assert cs.main("-LMIS,Mis", bad_name) == 1
+    assert cs.main("-I", fname, "-f", bad_name) == 1
+    assert cs.main("-LMIS,Mis", "-f", bad_name) == 1
+    fname.write_text("mis")
+    assert cs.main("-I", fname, bad_name) == 0
+    assert cs.main("-Lmis", bad_name) == 0
+    assert cs.main("-I", fname, "-f", bad_name) == 0
+    assert cs.main("-Lmis", "-f", bad_name) == 0
 
 
 def test_ignore_word_list(
@@ -335,6 +405,67 @@ def test_ignore_word_list(
     assert cs.main("-Labandonned,someword", "-Labilty", tmp_path) == 1
 
 
+@pytest.mark.parametrize(
+    ("content", "expected_error_count"),
+    [
+        # recommended form
+        ("abandonned abondon abilty  # codespell:ignore abondon", 2),
+        ("abandonned abondon abilty  // codespell:ignore abondon,abilty", 1),
+        (
+            "abandonned abondon abilty  /* codespell:ignore abandonned,abondon,abilty",
+            0,
+        ),
+        # ignore unused ignore
+        ("abandonned abondon abilty  # codespell:ignore nomenklatur", 3),
+        # wildcard form
+        ("abandonned abondon abilty  # codespell:ignore ", 0),
+        ("abandonned abondon abilty  # codespell:ignore", 0),
+        ("abandonned abondon abilty  # codespell:ignore\n", 0),
+        ("abandonned abondon abilty  # codespell:ignore\r\n", 0),
+        ("abandonned abondon abilty  # codespell:ignore  # noqa: E501\n", 0),
+        ("abandonned abondon abilty  # codespell:ignore # noqa: E501\n", 0),
+        ("abandonned abondon abilty  # codespell:ignore# noqa: E501\n", 0),
+        ("abandonned abondon abilty  # codespell:ignore, noqa: E501\n", 0),
+        ("abandonned abondon abilty  #codespell:ignore\n", 0),
+        # ignore these for safety
+        ("abandonned abondon abilty  # codespell:ignorenoqa: E501\n", 3),
+        ("abandonned abondon abilty  codespell:ignore\n", 3),
+        ("abandonned abondon abilty codespell:ignore\n", 3),
+        # ignore these as they aren't valid
+        ("abandonned abondon abilty  # codespell:igore\n", 4),
+        # showcase different comment markers
+        ("abandonned abondon abilty ' codespell:ignore\n", 0),
+        ('abandonned abondon abilty " codespell:ignore\n', 0),
+        ("abandonned abondon abilty ;; codespell:ignore\n", 0),
+        ("abandonned abondon abilty /* codespell:ignore */\n", 0),
+        # prose examples
+        (
+            "You could also use line based igore ( codespell:ignore ) to igore ",
+            0,
+        ),
+        ("You could also use line based igore (codespell:ignore) to igore ", 0),
+        (
+            "You could also use line based igore (codespell:ignore igore) to igore ",
+            0,
+        ),
+        (
+            "You could also use line based igore (codespell:ignore igare) to igore ",
+            2,
+        ),
+    ],
+)
+def test_inline_ignores(
+    tmpdir: pytest.TempPathFactory,
+    capsys: pytest.CaptureFixture[str],
+    content: str,
+    expected_error_count: int,
+) -> None:
+    d = str(tmpdir)
+    with open(op.join(d, "bad.txt"), "w") as f:
+        f.write(content)
+    assert cs.main(d) == expected_error_count
+
+
 def test_custom_regex(
     tmp_path: Path,
     capsys: pytest.CaptureFixture[str],
@@ -356,12 +487,35 @@ def test_exclude_file(
 ) -> None:
     """Test exclude file functionality."""
     bad_name = tmp_path / "bad.txt"
-    bad_name.write_bytes(b"1 abandonned 1\n2 abandonned 2\n")
-    assert cs.main(bad_name) == 2
+    # check all possible combinations of lines to ignore and ignores
+    combinations = "".join(
+        f"{n} abandonned {n}\n"
+        f"{n} abandonned {n}\r\n"
+        f"{n} abandonned {n} \n"
+        f"{n} abandonned {n} \r\n"
+        for n in range(1, 5)
+    )
+    bad_name.write_bytes(
+        (combinations + "5 abandonned 5\n6 abandonned 6").encode("utf-8")
+    )
+    assert cs.main(bad_name) == 18
     fname = tmp_path / "tmp.txt"
-    fname.write_bytes(b"1 abandonned 1\n")
-    assert cs.main(bad_name) == 2
+    fname.write_bytes(
+        b"1 abandonned 1\n"
+        b"2 abandonned 2\r\n"
+        b"3 abandonned 3 \n"
+        b"4 abandonned 4 \r\n"
+        b"6 abandonned 6\n"
+    )
+    assert cs.main(bad_name) == 18
     assert cs.main("-x", fname, bad_name) == 1
+    # comma-separated list of files
+    fname_dummy1 = tmp_path / "dummy1.txt"
+    fname_dummy1.touch()
+    fname_dummy2 = tmp_path / "dummy2.txt"
+    fname_dummy2.touch()
+    assert cs.main("-x", fname_dummy1, "-x", fname, "-x", fname_dummy2, bad_name) == 1
+    assert cs.main("-x", f"{fname_dummy1},{fname},{fname_dummy2}", bad_name) == 1
 
 
 def test_encoding(
@@ -402,7 +556,8 @@ def test_encoding(
     assert isinstance(result, tuple)
     code, stdout, stderr = result
     assert code == 0
-    assert not stdout and not stderr
+    assert not stdout
+    assert not stderr
     result = cs.main("-q", "0", fname, std=True, count=False)
     assert isinstance(result, tuple)
     code, stdout, stderr = result
@@ -439,9 +594,12 @@ def test_ignore(
     (subdir / "bad.txt").write_text("abandonned")
     assert cs.main(tmp_path) == 2
     assert cs.main("--skip=bad*", tmp_path) == 0
+    assert cs.main("--skip=whatever.txt,bad*,whatelse.txt", tmp_path) == 0
+    assert cs.main("--skip=whatever.txt,\n bad* ,", tmp_path) == 0
     assert cs.main("--skip=*ignoredir*", tmp_path) == 1
     assert cs.main("--skip=ignoredir", tmp_path) == 1
     assert cs.main("--skip=*ignoredir/bad*", tmp_path) == 1
+    assert cs.main(f"--skip={tmp_path}", tmp_path) == 0
     badjs = tmp_path / "bad.js"
     copyfile(badtxt, badjs)
     assert cs.main("--skip=*.js", goodtxt, badtxt, badjs) == 1
@@ -522,9 +680,9 @@ def test_check_hidden(
     #
     #         tmp_path
     #         ├── .abandonned
-    #         │   ├── .abandonned.txt
-    #         │   └── subdir
-    #         │       └── .abandonned.txt
+    #         │   ├── .abandonned.txt
+    #         │   └── subdir
+    #         │       └── .abandonned.txt
     #         └── .abandonned.txt
     #
     assert cs.main(tmp_path) == 0
@@ -554,9 +712,9 @@ def test_check_hidden(
     #
     #         tmp_path
     #         ├── .abandonned
-    #         │   ├── .abandonned.txt
-    #         │   └── subdir
-    #         │       └── .abandonned.txt
+    #         │   ├── .abandonned.txt
+    #         │   └── subdir
+    #         │       └── .abandonned.txt
     #         ├── .abandonned.txt
     #         └── subdir
     #             └── .abandonned
@@ -606,7 +764,7 @@ def _helper_test_case_handling_in_fixes(
     else:
         dictionary_name.write_text("adoptor->adopter, adaptor,\n")
 
-    # the mispelled word is entirely lowercase
+    # the misspelled word is entirely lowercase
     fname = tmp_path / "bad.txt"
     fname.write_text("early adoptor\n")
     result = cs.main("-D", dictionary_name, fname, std=True)
@@ -618,7 +776,7 @@ def _helper_test_case_handling_in_fixes(
     if reason:
         assert "reason" in stdout
 
-    # the mispelled word is capitalized
+    # the misspelled word is capitalized
     fname.write_text("Early Adoptor\n")
     result = cs.main("-D", dictionary_name, fname, std=True)
     assert isinstance(result, tuple)
@@ -629,7 +787,7 @@ def _helper_test_case_handling_in_fixes(
     if reason:
         assert "reason" in stdout
 
-    # the mispelled word is entirely uppercase
+    # the misspelled word is entirely uppercase
     fname.write_text("EARLY ADOPTOR\n")
     result = cs.main("-D", dictionary_name, fname, std=True)
     assert isinstance(result, tuple)
@@ -640,7 +798,7 @@ def _helper_test_case_handling_in_fixes(
     if reason:
         assert "reason" in stdout
 
-    # the mispelled word mixes lowercase and uppercase
+    # the misspelled word mixes lowercase and uppercase
     fname.write_text("EaRlY AdOpToR\n")
     result = cs.main("-D", dictionary_name, fname, std=True)
     assert isinstance(result, tuple)
@@ -1028,7 +1186,7 @@ def test_uri_regex_def() -> None:
         assert not uri_regex.findall(boilerplate % uri), uri
 
 
-def test_quiet_option_32(
+def test_quiet_level_32(
     tmp_path: Path,
     tmpdir: pytest.TempPathFactory,
     capsys: pytest.CaptureFixture[str],
@@ -1057,7 +1215,28 @@ def test_quiet_option_32(
     assert "setup.cfg" in stdout
 
 
-@pytest.mark.parametrize("kind", ("toml", "cfg"))
+def test_ill_formed_ini_config_file(
+    tmp_path: Path,
+    tmpdir: pytest.TempPathFactory,
+    capsys: pytest.CaptureFixture[str],
+) -> None:
+    d = tmp_path / "files"
+    d.mkdir()
+    conf = str(tmp_path / "setup.cfg")
+    with open(conf, "w") as f:
+        # It should contain but lacks a section.
+        f.write("foobar =\n")
+    args = ("--config", conf)
+
+    # Should not raise a configparser.Error exception.
+    result = cs.main(str(d), *args, std=True)
+    assert isinstance(result, tuple)
+    code, _, stderr = result
+    assert code == 78
+    assert "ill-formed config file" in stderr
+
+
+@pytest.mark.parametrize("kind", ["cfg", "cfg_multiline", "toml", "toml_list"])
 def test_config_toml(
     tmp_path: Path,
     capsys: pytest.CaptureFixture[str],
@@ -1079,30 +1258,47 @@ def test_config_toml(
     assert "bad.txt" in stdout
     assert "abandonned.txt" in stdout
 
-    if kind == "cfg":
+    if kind.startswith("cfg"):
         conffile = tmp_path / "setup.cfg"
         args = ("--config", conffile)
-        conffile.write_text(
-            """\
+        if kind == "cfg":
+            text = """\
 [codespell]
 skip = bad.txt, whatever.txt
 count =
 """
-        )
+        else:
+            assert kind == "cfg_multiline"
+            text = """\
+[codespell]
+skip = whatever.txt,
+   bad.txt ,
+   ,
+
+count =
+"""
+        conffile.write_text(text)
     else:
-        assert kind == "toml"
         if sys.version_info < (3, 11):
             pytest.importorskip("tomli")
         tomlfile = tmp_path / "pyproject.toml"
         args = ("--toml", tomlfile)
-        tomlfile.write_text(
-            """\
+        if kind == "toml":
+            text = """\
 [tool.codespell]
 skip = 'bad.txt,whatever.txt'
 check-filenames = false
 count = true
 """
-        )
+        else:
+            assert kind == "toml_list"
+            text = """\
+[tool.codespell]
+skip = ['bad.txt', 'whatever.txt']
+check-filenames = false
+count = true
+"""
+        tomlfile.write_text(text)
 
     # Should pass when skipping bad.txt or abandonned.txt
     result = cs.main(d, *args, std=True)
@@ -1135,3 +1331,39 @@ def FakeStdin(text: str) -> Generator[None, None, None]:
         yield
     finally:
         sys.stdin = oldin
+
+
+def run_codespell_stdin(
+    text: str,
+    args: Tuple[Any, ...],
+    cwd: Optional[Path] = None,
+) -> int:
+    """Run codespell in stdin mode and return number of lines in output."""
+    proc = subprocess.run(
+        ["codespell", *args, "-"],  # noqa: S603, S607
+        cwd=cwd,
+        input=text,
+        capture_output=True,
+        encoding="utf-8",
+        check=False,
+    )
+    output = proc.stdout
+    # get number of lines
+    return output.count("\n")
+
+
+def test_stdin(tmp_path: Path) -> None:
+    """Test running the codespell executable."""
+    input_file_lines = 4
+    text = ""
+    for _ in range(input_file_lines):
+        text += "abandonned\n"
+    for single_line_per_error in (True, False):
+        args: Tuple[str, ...] = ()
+        if single_line_per_error:
+            args = ("--stdin-single-line",)
+        # we expect 'input_file_lines' number of lines with
+        # --stdin-single-line and input_file_lines * 2 lines without it
+        assert run_codespell_stdin(
+            text, args=args, cwd=tmp_path
+        ) == input_file_lines * (2 - int(single_line_per_error))
